package extension

import (
	"context"

	"git.perx.ru/perxis/perxis-go/pkg/errors"
)

var (
	ErrInvalidAction = errors.New("invalid action")
)

// ActionHandler выполняет действие в url
type ActionHandler func(ctx context.Context, url *ActionURL, req *ActionRequest) (*ActionResponse, error)

// ActionRouter возвращает ActionHandler для указанного действия
type ActionRouter func(ctx context.Context, url *ActionURL, req *ActionRequest) (ActionHandler, error)

// Chain возвращает ActionRouter, который последовательно вызывает переданные ActionRouter
func Chain(chain ...ActionRouter) ActionRouter {
	return func(ctx context.Context, url *ActionURL, req *ActionRequest) (ActionHandler, error) {
		for _, f := range chain {
			if h, err := f(ctx, url, req); err == nil && h != nil {
				return h, nil
			}
		}
		return nil, ErrInvalidAction
	}
}

// Named возвращает ActionRouter, который возвращает ActionHandler для указанного действия по имени
func Named(name string, handler ActionHandler) ActionRouter {
	return func(ctx context.Context, url *ActionURL, req *ActionRequest) (ActionHandler, error) {
		if url.Action() == name {
			return handler, nil
		}
		return nil, ErrInvalidAction
	}
}
